#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "io.hh"

#define RAD 2.0
#define MAX_ITER 100

// check if a complex number is in the mandelbrot-set
int inMandelbrot(double c_real, double c_imag)
{
  double z_real = 0.0, z_imag = 0.0;
  double z_real_neu = 0.0, z_imag_neu = 0.0;

  int k = 0;
  for (k=0; k<MAX_ITER; k++)
  {
    if ( (z_real * z_real + z_imag * z_imag) > (RAD * RAD) )
    {
      return k;
    }
    else
    {
      z_real_neu = (z_real * z_real) - (z_imag * z_imag) + c_real;
      z_imag_neu = 2.0 * z_real * z_imag + c_imag;
    }

    z_real = z_real_neu;
    z_imag = z_imag_neu;
  }

  return -1;
}

// main function
int main(int argc, char** argv)
{
  const int asciiOut = 0;

  if (argc != 7)
  {
    printf("Usage: %s <xmin> <xmax> <xincrement> <ymin> <ymax> <yincrement>\n", argv[0]);
    exit(0);
  }

  double x1   = atof(argv[1]);
  double x2   = atof(argv[2]);
  double xinc = atof(argv[3]);
  double y1   = atof(argv[4]);
  double y2   = atof(argv[5]);
  double yinc = atof(argv[6]);
  printf("x1=%.2e x2=%2.e xinc=%.2e y1=%.2e y2=%2.e yinc=%.2e\n", x1, x2, xinc, y1, y2, yinc);

  // picture size
  const int nx = 1 + (int) (fabs(x2 - x1) / xinc);
  const int ny = 1 + (int) (fabs(y2 - y1) / yinc) ;

  // arrays for the RGB-colors of each pixel
  // RGB black: 0,0,0
  // RGB white: 255,255,255
  int* r   = (int*) malloc(nx * ny * sizeof(int));
  int* g   = (int*) malloc(nx * ny * sizeof(int));
  int* b   = (int*) malloc(nx * ny * sizeof(int));

  int i, j;
  for (i=0; i<nx; ++i)
  {
    for (j=0; j<ny; ++j)
    {
      const int index = i + j * nx;
      double re = x1 + xinc * i;
      double im = y1 + yinc * j;
      const int c = inMandelbrot(re, im);
      if ( c > -1 )
      {
        // white
        //r[index] = 255;
        //g[index] = 255;
        //b[index] = 255;

        // simple RGB coloring
        r[index] = (c % 32) * 7;
        g[index] = (c % 16) * 14;
        b[index] = (c % 128) * 2;

        if (asciiOut)
          printf(".");
      }
      else
      {
        r[index] = 0;
        g[index] = 0;
        b[index] = 0;

        if (asciiOut)
          printf("X");
      }
    }
    if (asciiOut)
      printf("\n");
  }

  // write ppm-file
  char* filename = "mandelbrot.ppm";
  int ierror = ppma_write(filename, nx, ny, r, g, b);

  return (0);
}
